from enum import Enum


class ValueRenderOption(str, Enum):
    """
    Determines how values should be rendered in the output.

    https://developers.google.com/sheets/api/reference/rest/v4/ValueRenderOption
    """

    FORMATTED_VALUE = "FORMATTED_VALUE"  #: Values will be calculated & formatted in the reply according to the cell's formatting.
    UNFORMATTED_VALUE = "UNFORMATTED_VALUE"  #: Values will be calculated, but not formatted in the reply.
    FORMULA = "FORMULA"  #: Values will not be calculated. The reply will include the formulas.



from enum import Enum


class DateTimeRenderOption(str, Enum):
    """
    Determines how dates should be rendered in the output.

    https://developers.google.com/sheets/api/reference/rest/v4/DateTimeRenderOption
    """

    SERIAL_NUMBER = "SERIAL_NUMBER"  #: Instructs date, time, datetime, and duration fields to be output as doubles in "serial number" format.
    FORMATTED_STRING = "FORMATTED_STRING"  #: Instructs date, time, datetime, and duration fields to be output as strings in their given number format.



from enum import Enum


class ValueInputOption(str, Enum):
    """
    Determines how input data should be interpreted.

    https://developers.google.com/sheets/api/reference/rest/v4/ValueInputOption
    """

    INPUT_VALUE_OPTION_UNSPECIFIED = "INPUT_VALUE_OPTION_UNSPECIFIED"  #: Default input value. This value must not be used.
    RAW = "RAW"  #: The values the user has entered will not be parsed and will be stored as-is.
    USER_ENTERED = "USER_ENTERED"  #: The values will be parsed as if the user typed them into the UI.



from typing import TYPE_CHECKING, ClassVar

from pydantic import Field
from tygle.apis.sheets.types.enums import (
    DateTimeRenderOption,
    Dimension,
    InsertDataOption,
    ValueInputOption,
    ValueRenderOption,
)
from tygle.base import Resource

if TYPE_CHECKING:
    from tygle.apis.sheets.rest.values import Values


class ValueRange(Resource):
    rest: ClassVar["Values"]
    """Data within a range of the spreadsheet.

    https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets.values#ValueRange
    """

    range: str = Field(title="range")
    major_dimension: Dimension = Field(alias="majorDimension", title="majorDimension")
    values: list[list] = Field(title="values")

    async def append(
        self,
        spreadsheet_id: str,
        range: str,
        /,
        *,
        value_input_option: ValueInputOption = ValueInputOption.RAW,
        insert_data_option: InsertDataOption = InsertDataOption.OVERWRITE,
        include_values_in_response: bool = False,
        response_value_render_option: ValueRenderOption = ValueRenderOption.FORMATTED_VALUE,
        response_date_time_render_option: DateTimeRenderOption = DateTimeRenderOption.SERIAL_NUMBER,
    ):
        """Chain to :meth:`.Values.append`.

        :param spreadsheet_id: The ID of the spreadsheet to update
        :type spreadsheet_id: int
        :param range: The `A1 notation <https://developers.google.com/sheets/api/guides/concepts#cell>` of a range to search for a logical table of data
        :type range: str
        :param value_input_option: How the input data should be interpreted, defaults to :attr:`.ValueInputOption.RAW`
        :type value_input_option: ValueInputOption, optional
        :param insert_data_option: How the input data should be inserted, defaults to :attr:`.InsertDataOption.OVERWRITE`
        :type insert_data_option: InsertDataOption, optional
        :param include_values_in_response: Determines if the update response should include the values of the cells that were appended, defaults to `False`
        :type include_values_in_response: bool, optional
        :param response_value_render_option: Determines how values in the response should be rendered, defaults to :attr:`.ValueRenderOption.FORMATTED_VALUE`
        :type response_value_render_option: ValueRenderOption, optional
        :param response_date_time_render_option: Determines how dates, times, and durations in the response should be rendered. This is ignored if :paramref:`.response_value_render_option` is :attr:`.ValueRenderOption.FORMATTED_VALUE`, defaults to :attr:`.DateTimeRenderOption.SERIAL_NUMBER`
        :type response_date_time_render_option: DateTimeRenderOption, optional
        """
        return self.rest.append(
            spreadsheet_id,
            range,
            self,
            value_input_option=value_input_option,
            insert_data_option=insert_data_option,
            include_values_in_response=include_values_in_response,
            response_value_render_option=response_value_render_option,
            response_date_time_render_option=response_date_time_render_option,
        )



from typing import Optional

from pydantic import BaseModel, Field

from .other import ExtendedValue


class CellData(BaseModel):
    """
    Data about a specific cell.

    https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets/cells#CellData
    """

    user_entered_value: Optional[ExtendedValue] = Field(None, alias="userEnteredValue")
    effective_value: Optional[ExtendedValue] = Field(None, alias="effectiveValue")
    formatted_value: Optional[str] = Field(None, alias="formattedValue")



# isort: skip_file
from .other import ErrorType, ErrorValue, ExtendedValue
from .cells import CellData
from .sheets import SheetProperties, GridRange, RowData, GridData, Sheet
from .spreadsheets import Spreadsheet



from typing import Optional

from pydantic import BaseModel, Field

from .cells import CellData


class SheetProperties(BaseModel):
    """
    Properties of a sheet.

    https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets/sheets#SheetProperties
    """

    sheet_id: int = Field(alias="sheetId")
    title: str = Field()


class GridRange(BaseModel):
    """
    A range on a sheet. All indexes are zero-based.
    Indexes are half open, i.e. the start index is inclusive and the end index is exclusive -- [startIndex, endIndex).
    Missing indexes indicate the range is unbounded on that side.

    https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets/other#GridRange
    """

    sheet_id: Optional[int] = Field(None, alias="sheetId")
    start_row_index: int = Field(alias="startRowIndex")
    end_row_index: int = Field(alias="endRowIndex")
    start_column_index: int = Field(alias="startColumnIndex")
    end_column_index: int = Field(alias="endColumnIndex")


class RowData(BaseModel):
    """
    Data about each cell in a row.

    https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets/sheets#RowData
    """

    values: list[CellData] = Field([])


class GridData(BaseModel):
    """
    Data in the grid, as well as metadata about the dimensions.

    https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets/sheets#GridData
    """

    start_row: Optional[int] = Field(None, alias="startRow")
    start_column: Optional[int] = Field(None, alias="startColumn")
    row_data: list[RowData] = Field(alias="rowData")


class Sheet(BaseModel):
    """
    A sheet in a spreadsheet.

    https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets/sheets#Sheet
    """

    properties: SheetProperties = Field()
    data: list[GridData] = Field()
    merges: list[GridRange] = Field()



from enum import Enum


# todo: move
class InsertDataOption(str, Enum):
    """
    Determines how existing data is changed when new data is input.

    https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets.values/append#InsertDataOption
    """

    OVERWRITE = "OVERWRITE"  #: The new data overwrites existing data in the areas it is written.
    INSERT_ROWS = "INSERT_ROWS"  #: Rows are inserted for the new data.



from aiogoogle import GoogleAPI
from tygle.base import API
from tygle.client import Client

from .rest import Spreadsheets


class Sheets(API):
    api_name = "sheets"
    api_version = "v4"

    def __init__(self, client: Client, api: GoogleAPI) -> None:
        super().__init__(client, api)
        self.spreadsheets = Spreadsheets(client, self.api)



from enum import Enum
from typing import Optional

from pydantic import BaseModel, Field


class ErrorType(str, Enum):
    """
    The type of error.

    https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets/other#ErrorType
    """

    ERROR_TYPE_UNSPECIFIED = (
        "ERROR_TYPE_UNSPECIFIED"  #: The default error type, do not use this.
    )
    ERROR = "ERROR"  #: Corresponds to the #ERROR! error.
    NULL_VALUE = "NULL_VALUE"  #: Corresponds to the #NULL! error.
    DIVIDE_BY_ZERO = "DIVIDE_BY_ZERO"  #: Corresponds to the #DIV/0 error.
    VALUE = "VALUE"  #: Corresponds to the #VALUE! error.
    REF = "REF"  #: Corresponds to the #REF! error.
    NAME = "NAME"  #: Corresponds to the #NAME? error.
    NUM = "NUM"  #: Corresponds to the #NUM! error.
    N_A = "N_A"  #: Corresponds to the #N/A error.
    LOADING = "LOADING"  #: Corresponds to the Loading... state.


class ErrorValue(BaseModel):
    """
    An error in a cell.

    https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets/other#ErrorValue
    """

    type: ErrorType = Field()
    message: str = Field()


class ExtendedValue(BaseModel):
    """
    The kinds of value that a cell in a spreadsheet can have.

    https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets/other#ExtendedValue
    """

    number_value: Optional[float] = Field(None, alias="numberValue")
    string_value: Optional[str] = Field(None, alias="stringValue")
    bool_value: Optional[bool] = Field(None, alias="boolValue")
    formula_value: Optional[str] = Field(None, alias="formulaValue")
    error_value: Optional[ErrorValue] = Field(None, alias="errorValue")



from typing import Optional

from pydantic import BaseModel, Field
from tygle.apis.sheets.types.resources.values import ValueRange


class UpdateValuesResponse(BaseModel):
    spreadsheet_id: str = Field(alias="spreadsheetId")
    updated_range: str = Field(alias="updatedRange")
    updated_rows: int = Field(alias="updatedRows")
    updated_columns: int = Field(alias="updatedColumns")
    updated_cells: int = Field(alias="updatedCells")
    updated_data: Optional[ValueRange] = Field(alias="updatedData")



from enum import Enum


class Dimension(str, Enum):
    """
    Indicates which dimension an operation should apply to.

    https://developers.google.com/sheets/api/reference/rest/v4/Dimension
    """

    DIMENSION_UNSPECIFIED = "DIMENSION_UNSPECIFIED"  #: The default value, do not use.
    ROWS = "ROWS"  #: Operates on the rows of a sheet.
    COLUMNS = "COLUMNS"  #: Operates on the columns of a sheet.



from pydantic import BaseModel, Field
from tygle.apis.sheets.types.responses import UpdateValuesResponse


class AppendValuesResponse(BaseModel):
    """
    The response when updating a range of values in a spreadsheet.

    https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets.values/append#response-body
    """

    spreadsheet_id: str = Field(alias="spreadsheetId")
    table_range: str = Field(alias="tableRange")
    updates: UpdateValuesResponse = Field()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['tygle_sheets',
 'tygle_sheets.rest',
 'tygle_sheets.types',
 'tygle_sheets.types.enums',
 'tygle_sheets.types.resources',
 'tygle_sheets.types.resources.spreadsheets',
 'tygle_sheets.types.resources.values',
 'tygle_sheets.types.responses',
 'tygle_sheets.types.responses.values']

package_data = \
{'': ['*']}

install_requires = \
['aiogoogle>=4.0.0,<5.0.0', 'pydantic>=1.9.1,<2.0.0', 'tygle>=0.1.0,<0.2.0']

setup_kwargs = {
    'name': 'tygle-sheets',
    'version': '0.1.0',
    'description': '',
    'long_description': '# tygle-sheets\n',
    'author': 'shmookoff',
    'author_email': 'shmookoff@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



from pydantic import BaseModel, Field
from tygle.apis.sheets.types.enums import Dimension
from tygle.base import Resource

from .sheets import Sheet


class SpreadsheetProperties(BaseModel):
    """
    Properties of a spreadsheet.

    https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets#spreadsheetproperties
    """

    title: str


class Spreadsheet(Resource):
    """
    Resource that represents a spreadsheet.

    https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets#resource:-spreadsheet
    """

    spreadsheet_id: str = Field(alias="spreadsheetId")
    properties: SpreadsheetProperties = Field(title="properties")
    sheets: list[Sheet] = Field(title="sheets")

    async def get_values(
        self,
        range: str,
        major_dimension: Dimension = Dimension.ROWS,
    ):
        return await self.client.sheets.spreadsheets.values.get(
            self.spreadsheet_id, range, major_dimension
        )



from typing import Optional

from aiogoogle import GoogleAPI
from pydantic import create_model
from tygle.apis.sheets.types.resources.spreadsheets import Spreadsheet
from tygle.base import REST, DataRequest
from tygle.client.client import Client

from .values import Values


class Spreadsheets(REST):
    def __init__(self, client: Client, parent: GoogleAPI) -> None:
        super().__init__(client, parent)
        self.values = Values(client, self.parent)  # todo move

    @property
    def Spreadsheet(self):
        return create_model("Spreadsheet", rest=self, __base__=Spreadsheet)

    async def get(
        self,
        spreadsheet_id: str,
        ranges: Optional[list[str]] = None,
        include_grid_data: bool = True,
    ) -> DataRequest["Spreadsheet"]:
        if ranges is None:
            ranges = []
        return DataRequest(
            self.client,
            self.parent.spreadsheets.get(
                spreadsheetId=spreadsheet_id,
                ranges=ranges,
                includeGridData=include_grid_data,
            ),
            Spreadsheet,
        )



from .date_time_render_option import DateTimeRenderOption
from .dimension import Dimension
from .insert_data_option import InsertDataOption
from .value_input_option import ValueInputOption
from .value_render_option import ValueRenderOption



from typing import Optional

from pydantic import create_model
from tygle.apis.sheets.types.enums import (
    DateTimeRenderOption,
    Dimension,
    InsertDataOption,
    ValueInputOption,
    ValueRenderOption,
)
from tygle.apis.sheets.types.resources.values import ValueRange
from tygle.apis.sheets.types.responses.values import AppendValuesResponse
from tygle.base import REST, DataRequest


class Values(REST):
    @property
    def ValueRange(self):
        return create_model("ValueRange", rest=self, __base__=ValueRange)

    def get(
        self,
        spreadsheet_id: str,
        range: str,
        /,
        *,
        major_dimension: Optional[Dimension] = None,
        value_render_option: Optional[ValueRenderOption] = None,
        date_time_render_option: Optional[DateTimeRenderOption] = None,
    ) -> DataRequest["ValueRange"]:
        """Returns a range of values from a spreadsheet. The caller must specify the spreadsheet ID and a range.

        https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets.values/get
        """
        return DataRequest(
            self.client,
            self.parent.spreadsheets.values.get(
                spreadsheetId=spreadsheet_id,
                range=range,
                majorDimension=major_dimension.value,
                valueRenderOption=value_render_option,
                dateTimeRenderOption=date_time_render_option,
            ),
            self.ValueRange,
        )

    def append(
        self,
        spreadsheet_id: str,
        range: str,
        /,
        value_range: "ValueRange",
        *,
        value_input_option: Optional[ValueInputOption] = None,
        insert_data_option: Optional[InsertDataOption] = None,
        include_values_in_response: Optional[bool] = None,
        response_value_render_option: Optional[ValueRenderOption] = None,
        response_date_time_render_option: Optional[DateTimeRenderOption] = None,
    ) -> DataRequest[AppendValuesResponse]:
        """Appends values to a spreadsheet.

        https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets.values/append
        """
        return DataRequest(
            self.client,
            self.parent.spreadsheets.values.append(
                spreadsheetId=spreadsheet_id,
                range=range,
                json=value_range.dict(),
                valueInputOption=value_input_option.value,
                insertDataOption=insert_data_option.value,
                includeValuesInResponse=include_values_in_response,
                responseValueRenderOption=response_value_render_option.value,
                responseDateTimeRenderOption=response_date_time_render_option.value,
            ),
            AppendValuesResponse,
        )
